;+
; NAME:
;  kepler1
; PURPOSE:   (one line only)
;  Solve Kepler's Equation (small eccentricity)
; DESCRIPTION:
;  Solve Kepler's Equation. Method by S. Mikkola (1987) Celestial
;     Mechanics, 40 , 329-334. 
; CATEGORY:
;  Celestial Mechanics
; CALLING SEQUENCE:
;  kepler1,m,ecc,eccanom
; INPUTS:
;  m    - Mean anomaly (radians)
;  ecc  - Eccentricity
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
;  eccanom - Eccentric Anomaly (radians)
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  2002/05/29 - Marc W. Buie, Lowell Observatory.  Ported from fortran routines
;    supplied by Larry Wasserman and Ted Bowell.
;-
pro kepler1,m,ecc,eccanom

   aux   =  4.d0*ecc+0.5d0
   alpha = (1.d0-ecc)/aux

   mx=m
   zz=where(mx gt !dpi,count)
   if count ne 0 then mx[zz]=mx[zz]-2.0d0*!dpi

   beta=mx/(2.d0*aux)
   aux=sqrt(beta^2+alpha^3)

   
   z=beta+aux
   zz=where(z le 0.0d0,count)
   if count gt 0 then z[zz]=beta[zz]-aux[zz]

   test=abs(z)^0.3333333333333333d0

   z =  test
   zz=where(z lt 0.0d0,count)
   if count gt 0 then z[zz] = -z[zz]

   s0=z-alpha/z
   s1=s0-(0.078d0*s0^5)/(1.d0+ecc)
   e0=mx+ecc*(3.d0*s1-4.d0*s1^3)

   se0=sin(e0)
   ce0=cos(e0)

   f  = e0-ecc*se0-mx
   f1 = 1.d0-ecc*ce0
   f2 = ecc*se0
   f3 = ecc*ce0
   f4 = -f2
   u1 = -f/f1
   u2 = -f/(f1+0.5d0*f2*u1)
   u3 = -f/(f1+0.5d0*f2*u2+.16666666666667d0*f3*u2*u2)
   u4 = -f/(f1+0.5d0*f2*u3+.16666666666667d0*f3*u3*u3+.041666666666667d0*f4*u3^3)

   eccanom=e0+u4

   zz = where(eccanom ge 2.0d0*!dpi,count)
   if count ne 0 then eccanom[zz]=eccanom[zz]-2.0d0*!dpi
   zz = where(eccanom lt 0.0d0,count)
   if count ne 0 then eccanom[zz]=eccanom[zz]+2.0d0*!dpi

end
