;+
; NAME:
;  sc_nearest
;
; PURPOSE:
;  Find the nearest N stars from a point in a star catalog.
;
; DESCRIPTION:
; CATEGORY:
;  Database
; CALLING SEQUENCE:
;  sc_nearest, db, ra, dec, radius, id, ra, dec, dra, ddec, mag
;
; INPUTS:
;  db     - handle to an open mysql ddatabase
;  ra     - ra of serach location
;  dec    - dec of serach location
;  radius - maximum disance from ra/dec to look
;
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  MAG_MIN - lower bound of star magnatude to return (eg. 7.5)
;  MAG_MAX - upper bound of star magnatude to return (eg. 8.0)
;  SCLASS - Spectral classification, as a regexp (eg. [ab]0 for a0 and b0 class)
;  MAX_COUNT - Maximum number of stars to return, defaults to 10
;  TBLNAME - Name of table to query in the DB
;
; OUTPUTS: Returns a series of arrays with up to n stars
;  id   - name of star
;  ra   - ra
;  dec  - dec
;  dra  - delta ra
;  ddec - delta dec
;  mag  - magnatude of stars
;
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  02/02/19, Writen by David Tucker
;-

pro sc_nearest, db, ra, dec, radius, $
    ret_id, ret_ra, ret_dec, ret_dra, ret_ddec, ret_mag, $
    MAG_MIN=mag_min, MAG_MAX=mag_max, SCLASS=sclass, $
    TBLNAME=tblname, MAX_COUNT=max_count

   if badpar(max_count,[0,1,2,3],0,CALLER='sc_nearest (MAX_COUNT) ',$
      default=10) then return
 
   ;query mysql db for bounding box
   sc_region, db, ra, dec, radius, tmptbl, $
      MAG_MIN=mag_min, MAG_MAX=mag_max, SCLASS=sclass, $
      TBLNAME=tblname

   ;query database
   query = 'SELECT id, ra, decl, dra, ddecl, mag FROM '+$
      tmptbl+' ORDER by tangsep ASC '+$
      'LIMIT '+STRING(max_count,FORMAT='(I)')+';'
   mysqlcmd, db, query, results, rcount

   ;parse data and return to caller
   results  = results[1:*]
   ret_id   = make_array(n_elements(results),/LONG)
   ret_ra   = make_array(n_elements(results),/DOUBLE)
   ret_dec  = make_array(n_elements(results),/DOUBLE)
   ret_dra  = make_array(n_elements(results),/DOUBLE)
   ret_ddec = make_array(n_elements(results),/DOUBLE)
   ret_mag  = make_array(n_elements(results),/DOUBLE)

   FOR i=0, n_elements(results)-1 DO BEGIN
      reads, results[i], t_id, t_ra, t_dec, t_dra, t_ddec, t_mag
      ret_id[i]   = t_id
      ret_ra[i]   = t_ra
      ret_dec[i]  = t_dec
      ret_dra[i]  = t_dra
      ret_ddec[i] = t_ddec
      ret_mag[i]  = t_mag
   ENDFOR

   ;close the temporary table
   query = 'DROP TABLE '+tmptbl+';'
   mysqlcmd, db, query, results, rcount

end

